#ifndef SLAREF_H
#define SLAREF_H  

/*
 * METRIC --- Mode expansion modeling in integrated optics / photonics
 * http://metric.computational-photonics.eu/ 
 */

/*
 * slaref.h
 * Reflection of plane waves from a dielectric multilayer stack 
 */

/* reflection of a plane wave from a dielectric multilayer stack:
   compute reflection + transmission coefficients, assemble the stationary
   electromagnetic field:
   wg: the multilayer structure, interface normal along x-axis,
       needs at least 1 inner layer
   pol: input polarization, use TM for parallel polarization, 
                                TE for orthogonal polarization
   theta: incidence angle, in radiants
   in: intensity of the input wave, with respect to the surface (y-z-plane)
   ref (output): the reflected intensity, reflectivity for in=1
   trans (output): the transmitted intensity, transmittivity for in=1 
   fld (output): the electromagnetic field, basic wave profiles 
   amp (output): their amplitudes, Cvector(>=2) */
void mlref(Waveguide wg, Polarization pol, double theta, double in,
           double& ref, double& trans, ModeArray& fld, Cvector& amp);
/*  rc, tc (output): complex relative amplitude coefficients 
                     of reflected, and transmitted waves */
void mlref(Waveguide wg, Polarization pol, double theta, 
           double in, 
           double& ref, Complex& rc,
           double& trans, Complex& tc, 
           ModeArray& fld, Cvector& amp);


#endif  // SLAREF_H   
